@@ -455,7 +455,7 @@ impl<'lua> Table<'lua> {
         }
     }
 
-    #[cfg(any(feature = "async", feature = "serialize"))]
+    #[cfg(any(feature = "serialize"))]
     pub(crate) fn raw_sequence_values_by_len<V: FromLua<'lua>>(
         self,
         len: Option<Integer>,
