@@ -1,12 +1,14 @@
 use std::cell::Cell;
 use std::rc::Rc;
+use std::sync::Arc;
 
 use mlua::{
-    AnyUserData, Error, Function, Lua, MetaMethod, Result, String, UserData, UserDataMethods,
+    AnyUserData, Error, Function, Lua, MetaMethod, Result, String, UserData, UserDataFields,
+    UserDataMethods,
 };
 
 #[test]
-fn scope_func() -> Result<()> {
+fn test_scope_func() -> Result<()> {
     let lua = Lua::new();
 
     let rc = Rc::new(Cell::new(0));
@@ -25,7 +27,10 @@ fn scope_func() -> Result<()> {
     assert_eq!(Rc::strong_count(&rc), 1);
 
     match lua.globals().get::<_, Function>("bad")?.call::<_, ()>(()) {
-        Err(Error::CallbackError { .. }) => {}
+        Err(Error::CallbackError { ref cause, .. }) => match *cause.as_ref() {
+            Error::CallbackDestructed => {}
+            ref err => panic!("wrong error type {:?}", err),
+        },
         r => panic!("improper return for destructed function: {:?}", r),
     };
 
@@ -33,81 +38,7 @@ fn scope_func() -> Result<()> {
 }
 
 #[test]
-fn scope_drop() -> Result<()> {
-    let lua = Lua::new();
-
-    struct MyUserdata(Rc<()>);
-    impl UserData for MyUserdata {
-        fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
-            methods.add_method("method", |_, _, ()| Ok(()));
-        }
-    }
-
-    let rc = Rc::new(());
-
-    lua.scope(|scope| {
-        lua.globals()
-            .set("static_ud", scope.create_userdata(MyUserdata(rc.clone()))?)?;
-        assert_eq!(Rc::strong_count(&rc), 2);
-        Ok(())
-    })?;
-    assert_eq!(Rc::strong_count(&rc), 1);
-
-    match lua.load("static_ud:method()").exec() {
-        Err(Error::CallbackError { ref cause, .. }) => match cause.as_ref() {
-            Error::CallbackDestructed => {}
-            e => panic!("expected CallbackDestructed, got {:?}", e),
-        },
-        r => panic!("improper return for destructed userdata: {:?}", r),
-    };
-
-    let static_ud = lua.globals().get::<_, AnyUserData>("static_ud")?;
-    match static_ud.borrow::<MyUserdata>() {
-        Ok(_) => panic!("borrowed destructed userdata"),
-        Err(Error::UserDataDestructed) => {}
-        Err(e) => panic!("expected UserDataDestructed, got {:?}", e),
-    }
-
-    // Check non-static UserData drop
-    struct MyUserDataRef<'a>(&'a Cell<i64>);
-
-    impl<'a> UserData for MyUserDataRef<'a> {
-        fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
-            methods.add_method("inc", |_, data, ()| {
-                data.0.set(data.0.get() + 1);
-                Ok(())
-            });
-        }
-    }
-
-    let i = Cell::new(1);
-    lua.scope(|scope| {
-        lua.globals().set(
-            "nonstatic_ud",
-            scope.create_nonstatic_userdata(MyUserDataRef(&i))?,
-        )
-    })?;
-
-    match lua.load("nonstatic_ud:inc(1)").exec() {
-        Err(Error::CallbackError { ref cause, .. }) => match cause.as_ref() {
-            Error::CallbackDestructed => {}
-            e => panic!("expected CallbackDestructed, got {:?}", e),
-        },
-        r => panic!("improper return for destructed userdata: {:?}", r),
-    };
-
-    let nonstatic_ud = lua.globals().get::<_, AnyUserData>("nonstatic_ud")?;
-    match nonstatic_ud.borrow::<MyUserDataRef>() {
-        Ok(_) => panic!("borrowed destructed userdata"),
-        Err(Error::UserDataDestructed) => {}
-        Err(e) => panic!("expected UserDataDestructed, got {:?}", e),
-    }
-
-    Ok(())
-}
-
-#[test]
-fn scope_capture() -> Result<()> {
+fn test_scope_capture() -> Result<()> {
     let lua = Lua::new();
 
     let mut i = 0;
@@ -125,7 +56,7 @@ fn scope_capture() -> Result<()> {
 }
 
 #[test]
-fn outer_lua_access() -> Result<()> {
+fn test_scope_outer_lua_access() -> Result<()> {
     let lua = Lua::new();
 
     let table = lua.create_table()?;
@@ -140,7 +71,42 @@ fn outer_lua_access() -> Result<()> {
 }
 
 #[test]
-fn scope_userdata_methods() -> Result<()> {
+fn test_scope_userdata_fields() -> Result<()> {
+    struct MyUserData<'a>(&'a Cell<i64>);
+
+    impl<'a> UserData for MyUserData<'a> {
+        fn add_fields<'lua, F: UserDataFields<'lua, Self>>(fields: &mut F) {
+            fields.add_field_method_get("val", |_, data| Ok(data.0.get()));
+            fields.add_field_method_set("val", |_, data, val| {
+                data.0.set(val);
+                Ok(())
+            });
+        }
+    }
+
+    let lua = Lua::new();
+
+    let i = Cell::new(42);
+    let f: Function = lua
+        .load(
+            r#"
+            function(u)
+                assert(u.val == 42)
+                u.val = 44
+            end
+        "#,
+        )
+        .eval()?;
+
+    lua.scope(|scope| f.call::<_, ()>(scope.create_nonstatic_userdata(MyUserData(&i))?))?;
+
+    assert_eq!(i.get(), 44);
+
+    Ok(())
+}
+
+#[test]
+fn test_scope_userdata_methods() -> Result<()> {
     struct MyUserData<'a>(&'a Cell<i64>);
 
     impl<'a> UserData for MyUserData<'a> {
@@ -181,7 +147,7 @@ fn scope_userdata_methods() -> Result<()> {
 }
 
 #[test]
-fn scope_userdata_functions() -> Result<()> {
+fn test_scope_userdata_functions() -> Result<()> {
     struct MyUserData<'a>(&'a i64);
 
     impl<'a> UserData for MyUserData<'a> {
@@ -223,7 +189,7 @@ fn scope_userdata_functions() -> Result<()> {
 }
 
 #[test]
-fn scope_userdata_mismatch() -> Result<()> {
+fn test_scope_userdata_mismatch() -> Result<()> {
     struct MyUserData<'a>(&'a Cell<i64>);
 
     impl<'a> UserData for MyUserData<'a> {
@@ -273,3 +239,120 @@ fn scope_userdata_mismatch() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_scope_userdata_drop() -> Result<()> {
+    let lua = Lua::new();
+
+    struct MyUserData(Rc<()>);
+
+    impl UserData for MyUserData {
+        fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
+            methods.add_method("method", |_, _, ()| Ok(()));
+        }
+    }
+
+    struct MyUserDataArc(Arc<()>);
+
+    impl UserData for MyUserDataArc {}
+
+    let rc = Rc::new(());
+    let arc = Arc::new(());
+    lua.scope(|scope| {
+        let ud = scope.create_userdata(MyUserData(rc.clone()))?;
+        ud.set_user_value(MyUserDataArc(arc.clone()))?;
+        lua.globals().set("ud", ud)?;
+        assert_eq!(Rc::strong_count(&rc), 2);
+        assert_eq!(Arc::strong_count(&arc), 2);
+        Ok(())
+    })?;
+
+    lua.gc_collect()?;
+    assert_eq!(Rc::strong_count(&rc), 1);
+    assert_eq!(Arc::strong_count(&arc), 1);
+
+    match lua.load("ud:method()").exec() {
+        Err(Error::CallbackError { ref cause, .. }) => match cause.as_ref() {
+            Error::CallbackDestructed => {}
+            err => panic!("expected CallbackDestructed, got {:?}", err),
+        },
+        r => panic!("improper return for destructed userdata: {:?}", r),
+    };
+
+    let ud = lua.globals().get::<_, AnyUserData>("ud")?;
+    match ud.borrow::<MyUserData>() {
+        Ok(_) => panic!("succesfull borrow for destructed userdata"),
+        Err(Error::UserDataDestructed) => {}
+        Err(err) => panic!("improper borrow error for destructed userdata: {:?}", err),
+    }
+
+    match ud.get_metatable() {
+        Ok(_) => panic!("successful metatable retrieval of destructed userdata"),
+        Err(Error::UserDataDestructed) => {}
+        Err(err) => panic!(
+            "improper metatable error for destructed userdata: {:?}",
+            err
+        ),
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_scope_nonstatic_userdata_drop() -> Result<()> {
+    let lua = Lua::new();
+
+    struct MyUserData<'a>(&'a Cell<i64>, Arc<()>);
+
+    impl<'a> UserData for MyUserData<'a> {
+        fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
+            methods.add_method("inc", |_, data, ()| {
+                data.0.set(data.0.get() + 1);
+                Ok(())
+            });
+        }
+    }
+
+    struct MyUserDataArc(Arc<()>);
+
+    impl UserData for MyUserDataArc {}
+
+    let i = Cell::new(1);
+    let arc = Arc::new(());
+    lua.scope(|scope| {
+        let ud = scope.create_nonstatic_userdata(MyUserData(&i, arc.clone()))?;
+        ud.set_user_value(MyUserDataArc(arc.clone()))?;
+        lua.globals().set("ud", ud)?;
+        lua.load("ud:inc()").exec()?;
+        assert_eq!(Arc::strong_count(&arc), 3);
+        Ok(())
+    })?;
+
+    lua.gc_collect()?;
+    assert_eq!(Arc::strong_count(&arc), 1);
+
+    match lua.load("ud:inc()").exec() {
+        Err(Error::CallbackError { ref cause, .. }) => match cause.as_ref() {
+            Error::CallbackDestructed => {}
+            err => panic!("expected CallbackDestructed, got {:?}", err),
+        },
+        r => panic!("improper return for destructed userdata: {:?}", r),
+    };
+
+    let ud = lua.globals().get::<_, AnyUserData>("ud")?;
+    match ud.borrow::<MyUserData>() {
+        Ok(_) => panic!("succesfull borrow for destructed userdata"),
+        Err(Error::UserDataDestructed) => {}
+        Err(err) => panic!("improper borrow error for destructed userdata: {:?}", err),
+    }
+    match ud.get_metatable() {
+        Ok(_) => panic!("successful metatable retrieval of destructed userdata"),
+        Err(Error::UserDataDestructed) => {}
+        Err(err) => panic!(
+            "improper metatable error for destructed userdata: {:?}",
+            err
+        ),
+    }
+
+    Ok(())
+}
