@@ -800,6 +800,17 @@ fn test_drop_registry_value() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn test_replace_registry_value() -> Result<()> {
+    let lua = Lua::new();
+
+    let key = lua.create_registry_value::<i32>(42)?;
+    lua.replace_registry_value(&key, "new value")?;
+    assert_eq!(lua.registry_value::<String>(&key)?, "new value");
+
+    Ok(())
+}
+
 #[test]
 fn test_lua_registry_hash() -> Result<()> {
     let lua = Lua::new();
@@ -1256,3 +1267,24 @@ fn test_warnings() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+#[cfg(feature = "luajit")]
+#[should_panic]
+fn test_luajit_cdata() {
+    let lua = unsafe { Lua::unsafe_new() };
+    let _v: Result<Value> = lua
+        .load(
+            r#"
+        local ffi = require("ffi")
+        ffi.cdef[[
+            void *malloc(size_t size);
+            void free(void *ptr);
+        ]]
+        local ptr = ffi.C.malloc(1)
+        ffi.C.free(ptr)
+        return ptr
+    "#,
+        )
+        .eval();
+}
